###############################################################
#                                                             #
# Template code to compute stresses in a half plane subject   #
# to a distributed load on its surface.                       #
#                                                             #
# "Contact Mechanics and Elements of Tribology" (CMET) course #
#                                                             #
# V.A. Yastrebov, CNRS                                        #
# MINES Paris, PSL University, Centre des materiaux           #
# Feb 2021-Feb 2023                                           #
#                                                             #
# Licence: CC0                                                #
#                                                             #
###############################################################

import sys
import numpy as np
import scipy.integrate as integrate
import matplotlib.pyplot as plt
import matplotlib 
from matplotlib import cm

matplotlib.rcParams.update(matplotlib.rcParamsDefault)
matplotlib.rcParams['figure.figsize'] = 10,8
matplotlib.rcParams.update({'font.size': 14})
# plt.rc('text', usetex=True)
# plt.rc('font', family='serif')

# Function to plot stress fields
def PlotField(X: np.dtype(float),Y: np.dtype(float),Z: np.dtype(float),label: str,filename: str) -> None:
  fig,ax = plt.subplots()
  ax.set_aspect(1)
  plt.xlabel("x/a")
  plt.ylabel("y/a")

  lev = np.linspace(np.min(Z/p0),np.max(Z/p0),NumColors)

  cs = ax.contourf(X/a, Y/a, Z/p0, levels=lev, cmap=cm.rainbow)
  ax.contour(X/a,Y/a,Z/p0,levels=lev,colors="k")
  cbar = fig.colorbar(cs)
  cbar.ax.set_ylabel(label)
  fig.savefig(filename)
  plt.show()

# Contact pressure is distributed from -a to a
a = 1.
# Poisson's ratio & Young's modulus
v = 0.3
E = 1e3
# Pressure constant
p0 = 10.

### HELPER on Python syntaxis ###
# product a*b
# power   a**b
# pi      np.pi
# square root np.sqrt(x)
#################################

# Pressure distribution
def pres(x):
  if abs(x) <= a:
    return p0 
  else:
    return 0.

# Integrands
def gx(x,s):
  return 1/E * x * pres(s)
def gy(x,s):
  return 1/E * x**2 * pres(s)

# Read arguments.
# Warning: if you do not know how to provide your code with argument or simply to accelerate it,
# you can comment the lines below until "# EO COMMENT" and uncomment two lines below "L=4*a,N=10"
if len(sys.argv) != 3:
  print("I need 2 arguments: 1 - box size (double), 2 - number of output points per side (int).")
  exit(1)
# Box size
L = float(sys.argv[1])
# Number of output points per side
N = int(sys.argv[2])
# EO COMMENT
# L = 4*a
# N = 10

# Stress arrays
u_x = np.zeros(N)
u_y = np.zeros(N)

# Discretization
dl = L/float(N)
# Small values to integrate singular integral
eps = a*1e-3


# Evaluate integrals on the grid x in (-L/2,L/2) fill numpy vectors of displacements and save all data in a file
output = open("displacement.data","w")
output.write("# [1] x [2] u_x [3] u_y\n")

x=-L/2.+dl/2
# Loop over x points
for i in range(N): 
    if abs(x) < a:
# Evaluate integrals
        u_x[i] = integrate.quad(lambda s: gx(x,s), -a, x)[0] - integrate.quad(lambda s: gx(x,s), x, a)[0]
        u_y[i] = integrate.quad(lambda s: gy(x,s), -a, a, points=x)[0] 
    else:
        u_x[i] = integrate.quad(lambda s: gx(x,s), -a, a)[0] * x/abs(x)
        u_y[i] = integrate.quad(lambda s: gy(x,s), -a, a,  points=x)[0] 
    output.write(str(x)+" "+str(u_x[i])+" "+str(u_y[i])+"\n")
    x += dl
output.close()    

# Plot results: u_x and u_y
x = np.linspace(-L/2. + dl/2., L/2. - dl/2., N)

fig,ax = plt.subplots()
ax.grid(True)
plt.xlabel("x/a")
plt.ylabel("Ux,Uy")

plt.plot(x,u_x,"o-",color="r",label="Ux")
# We assume that u_y = 0 at the limits of the simulation box
plt.plot(x,-u_y + np.min(u_y),"v-",color="navy",label="Uy")
plt.axvline(-1,ls="--",color="k")
plt.axvline(1,ls="--",color="k")
plt.legend(loc="best")
fig.savefig("Displacements.pdf")
plt.show()